﻿using System;
 
 
using System.Text;
using System.Collections;

namespace STMBench7.Compatability
{
    /**
     * Originally because Bartok lacked LinkedList<T>, now because
     * I don't see a standard nongeneric linked list.  --Ryan.
     */
    public class LinkedList2 : ICollection
    {
        private Node head, tail;

        public Node First
        {
            get { return head; }
        }


        [BartokSTMExtensions.NoLogTrans] //allocation that does not escape "this"
        public LinkedList2()
        {
        }

        public LinkedList2(IEnumerable c)
        {
            foreach (Object e in c) //need to log the reads for the copied values
                AddLastNoLog(e);
        }

        [BartokSTMExtensions.NoLogTrans] //special case during init
        private void AddLastNoLog(Object e)
        {
            Node n = new Node();
            n.prev = tail;
            n.value = e;

            if (tail != null) tail.next = n;
            tail = n;
            if (head == null) head = n;
        }

        public void AddLast(Object e)
        {
            Node n = new Node();
            n.prev = tail;
            n.value = e;

            if (tail != null) tail.next = n;
            tail = n;
            if (head == null) head = n;
        }
        public void AddFirst(Object e)
        {
            Node n = new Node();
            n.next = head;
            n.value = e;

            if (head != null) head.prev = n;
            head = n;
            if (tail == null) tail = n;
        }
        public void Add(Object e)
        {
            AddLast(e);
        }
        public bool Remove(Object e)
        {
            return RemoveFirst(e);
        }
        public bool RemoveFirst(Object e)
        {
            Node n = head;
            while (n != null)
            {
                if (  Benchmark.TransactionSafeEquals(n.value,e) )
                {
                    if (n.prev == null) //n is head
                        head = n.next;
                    else
                        n.prev.next = n.next;

                    if (n.next == null) //n is tail
                        tail = n.prev;
                    else
                        n.next.prev = n.prev;

                    return true;
                }
                n = n.next;
            }
            return false;
        }

        public void RemoveFirst()
        {
            Node n = head.next;

            if (n == null)
            {
                head = tail = null;
                return;
            }

            n.prev = null;
            head = n;
            return;
        }

        public void Clear()
        {
            head = tail = null;
        }
        public bool Contains(Object e)
        {
            Node n = head;
            while (n != null)
            {
                if (Benchmark.TransactionSafeEquals(e,n.value)) return true;
                n = n.next;
            }
            return false;
        }
        public bool IsReadOnly
        {
            get { return false; }
        }
        public void CopyTo(Array array, int offset)
        {
            throw new NotImplementedException();
        }
        public bool IsSynchronized
        {
            get
            {
                throw new NotImplementedException();
            }
        }
        public object SyncRoot
        {
            get
            {
                throw new NotImplementedException();
            }
        }

        public int Count
        {
            get
            {
                int c = 0;
                Node n = head;
                while (n != null)
                {
                    c++;
                    n = n.next;
                }
                return c;
            }

        }

        class LinkedList2Enumerator : IEnumerator
        {
            LinkedList2 l;
            Node n;
            Node before = new Node(); //a sential value
            public LinkedList2Enumerator(LinkedList2 ll)
            {
                l = ll;
                Reset();
            }
            
            public bool MoveNext()
            {
                if (n == null) return false;

                if (n == before) n = l.head;
                else n = n.next;

                return n != null;
            }
            public void Reset()
            {
                n = before;
            }
            public void Dispose()
            {
                n = null;
                l = null;
            }
            
            object IEnumerator.Current
            {
                get { 
                    return n==null ? null : n.value;
                }
            }
        }

        public IEnumerator GetEnumerator()
        {
            return new LinkedList2Enumerator(this);
        }
    }

    public class Node
    {
        public Node next, prev;
        public Object value;

        public Object Value
        {
            get
            {
                return value;
            }
        }
    }
}
