﻿using System;
 
 
using System.Text;

using STMBench7.Annotations;
using STMBench7.Core;
using STMBench7.Backend;
using STMBench7.Compatability;

namespace STMBench7.Correctness.Invariants
{
	/**
	 * Test of invariants of an atomic part.
	 */
	[Immutable]
	[ThreadLocal]
	public class AtomicPartTest : InvariantTest {

		public static void CheckInvariants(AtomicPart part, bool initial, CompositePart component, 
				HashtableSet/*<AtomicPart>*/ allParts, TraversedObjects traversedObjects) {

			traversedObjects.atomicParts.Add(part);
			
			DesignObjTest.CheckInvariants(part, initial, Parameters.MaxAtomicParts,
					Parameters.MinAtomicDate, Parameters.MaxAtomicDate);
		
			int id = part.GetId(), x = part.GetX(), y = part.GetY();
			if(Math.Abs(x - y) != 1)
				ReportError(part, id, "inconsistent x and y attributes: x = " + x + ", y = " + y);
			
			if(part.GetPartOf() != component)
				ReportError(part, id, "invalid reference to CompositePart parent");
			
			ImmutableCollection/*<Connection>*/ to = part.GetToConnections(), from = part.GetFromConnections();
			if(to.Size() != Parameters.NumConnPerAtomic)
				ReportError(part, id, "to", Parameters.NumConnPerAtomic, to.Size());
			
			foreach(Connection connection in from) {
				ConnectionTest.CheckInvariants(connection, part);
				
				ImmutableCollection/*<Connection>*/ sourceToSet = connection.GetSource().GetToConnections();
				bool foundMyself = false;
				foreach(Connection outConn in sourceToSet)
					if(outConn.GetSource() == part) {
						foundMyself = true;
						break;
					}
				if(! foundMyself)
					ReportError(part, id, "inconsistent 'from' set");
			}
			
			foreach(Connection connection in to) {
				ConnectionTest.CheckInvariants(connection, part);
				
				allParts.Add(connection.GetDestination());
			}
		}
	}

}
