﻿using System;
 
 
using System.Text;

using STMBench7.Annotations;
using STMBench7.Core;
using STMBench7.Backend;
using STMBench7.Compatability;

namespace STMBench7.Correctness.Invariants
{
	/**
	 * Test of invariants of a composite part.
	 */
	[Immutable]
	[ThreadLocal]
	public class CompositePartTest : InvariantTest {

		public static void CheckInvariants(CompositePart component, bool initial, BaseAssembly aParent, 
				TraversedObjects traversedObjects) {

			traversedObjects.components.Add(component);

			Console.Error.Write("Component " + traversedObjects.components.Count + "\r");
				
			DesignObjTest.CheckInvariants(component, initial, Parameters.MaxCompParts, 0, int.MaxValue - 1); 
			
			int id = component.GetId(), buildDate = component.GetBuildDate();
			int minOldDate = Parameters.MinOldCompDate, maxOldDate = Parameters.MaxOldCompDate,
				minYoungDate = Parameters.MinYoungCompDate, maxYoungDate = Parameters.MaxYoungCompDate;
			if(! initial) {
				if(minOldDate % 2 == 0) minOldDate--;
				if(maxOldDate % 2 != 0) maxOldDate++;
				if(minYoungDate % 2 == 0) minYoungDate--;
				if(maxYoungDate % 2 != 0) maxYoungDate++;
			}
			if( (buildDate < minOldDate|| buildDate > maxOldDate) &&
					(buildDate < minYoungDate || buildDate > maxYoungDate) )
				ReportError(component, id, "wrong buildDate (" + buildDate + ")");

			ImmutableCollection/*<BaseAssembly>*/ usedIn = component.GetUsedIn();
			if(aParent != null && !usedIn.Contains(aParent))
				ReportError(component, id, "a BaseAssembly parent not in set usedIn");
			
			foreach(BaseAssembly assembly in usedIn) {
				if(! assembly.GetComponents().Contains(component))
					ReportError(assembly, assembly.GetId(), "a child CompositePart not in set of components");
			}
			
			DocumentationTest.CheckInvariants(component.GetDocumentation(), component, traversedObjects);
			
			LargeSet/*<AtomicPart>*/ parts = component.GetParts();
			if(! parts.Contains(component.GetRootPart()))
				ReportError(component, id, "rootPart not in set of parts");
			
			//TreeSet<AtomicPart> allConnectedParts = new TreeSet<AtomicPart>();
            HashtableSet/*<AtomicPart>*/ allConnectedParts = new HashtableSet/*<AtomicPart>*/();
            foreach(AtomicPart part in parts)
				AtomicPartTest.CheckInvariants(part, initial, component, allConnectedParts, traversedObjects);
			
			foreach(AtomicPart part in allConnectedParts)
				if(! parts.Contains(part)) ReportError(component, id, "a graph-connected AtomicPart not in set of all parts");
		}
	}

}
