﻿using System;
 
 
using System.Text;

using STMBench7.Annotations;
using STMBench7.Core;

namespace STMBench7.Correctness.Invariants
{
	/**
	 * Test of common invariants of elements inherited from a design object.
	 */
	[Immutable]
	[ThreadLocal]
	public class DesignObjTest : InvariantTest {

		public static void CheckInvariants(DesignObj obj, bool initial, int maxId, int minBuildDate, int maxBuildDate) {
			int id = obj.GetId();
			if(id < 1 || id > maxId) ReportError(obj, id, "id", 1, maxId, id);
			
			String type = obj.GetType();
			if(! CheckType(type)) ReportError(obj, id, "type", "type #(num)", type);
			
			int buildDate = obj.GetBuildDate();
			if(! initial) {
				if(minBuildDate % 2 == 0) minBuildDate--;
				if(maxBuildDate % 2 != 0) maxBuildDate++;
			}
			if(buildDate < minBuildDate || buildDate > maxBuildDate)
				ReportError(obj, id, "buildDate", minBuildDate, maxBuildDate, buildDate);
		}
		
		public static bool CheckType(String type) {
			if(! type.StartsWith("type #")) return false;
			
			try {
				int typeNum = int.Parse(type.Substring("type #".Length));
				if(typeNum < 0 || typeNum >= Parameters.NumTypes) return false;
				
			}
			catch(FormatException) {
				return false;
			}
			
			return true;
		}
	}

}
