﻿using System;
 
 
using System.Text;

using STMBench7.Annotations;
using STMBench7.Core;

namespace STMBench7.Correctness.Invariants
{
	/**
	 * Test of invariants of a documentation element.
	 */
	[Immutable]
	[ThreadLocal]
	public class DocumentationTest : InvariantTest {

		public static void CheckInvariants(Document documentation,
				CompositePart component, TraversedObjects traversedObjects) {

			traversedObjects.documents.Add(documentation);
			
			int id = documentation.GetDocumentId();
			if(id < 1 || id > Parameters.MaxCompParts)
				ReportError(documentation, id, "id", 1, Parameters.MaxCompParts, id);
			
			if(documentation.GetCompositePart() != component)
				ReportError(documentation, id, "invalid reference to CompositePart parent");
			
			String title = documentation.GetTitle(),
				titleShouldBe = "Composite Part #" + component.GetId();
			if(! title.Equals(titleShouldBe))
				ReportError(documentation, id, "title", titleShouldBe, title);
			
			String text = documentation.GetText();
			if(!text.StartsWith("I am the documentation for composite part #" + component.GetId()) &&
					!text.StartsWith("This is the documentation for composite part #" + component.GetId()) )
				ReportError(documentation, id, "text (prefix)", "I am / This is the documentation for composite part #...",
						text.Substring(0, 30));
		}
	}

}
