﻿using System;
 
 
using System.Text;

using STMBench7.Annotations;
using STMBench7.Core;
using System.Collections;

namespace STMBench7.Correctness.Opacity
{
	/**
	 * Replays sequentially a concurrent execution. Used to check
	 * whether a given concurrent execution ensures opacity, i.e., 
	 * whether the synchronization method used in the execution
	 * was correctly synchonizing threads during this execution.
	 */
	[NonAtomic]
	public class SequentialReplayThread : BenchThread {

		public SequentialReplayThread(Setup setup, double[] operationCDF,
                ArrayList/*<ReplayLogEntry>*/ replayLog)
            : base(setup, operationCDF)
        {
			this.replayLog = replayLog;
			ThreadRandom.StartSequentialReplayPhase();
		}
		
		public override void Run() {
			int opNum = 1, numOfOps = replayLog.Count;
			foreach(ReplayLogEntry entry in replayLog) {
				Console.Error.Write("Operation " + (opNum++) + " out of " + numOfOps + "\r");
				short threadNum = entry.threadNum;
				ThreadRandom.SetVirtualThreadNumber(threadNum);
				
				int operationNumber = GetNextOperationNumber();
				int result = 0;
				bool failed = false;
				
				try {
					result = operations[operationNumber].Execute();
				}
				catch(OperationFailedException) {
					failed = true;
				}
				
				if(result != entry.result || failed != entry.failed) {
					String opName = OperationId.values()[operationNumber].ToString();
					throw new RuntimeError("Different operation result in the sequential execution (" +
							"operation " + opName + "): " +
							"Sequential: result = " + result + ", failed = " + failed + ". " + 
							"Concurrent: result = " + entry.result + ", failed = " + entry.failed + ".");
				}
			}
			Console.Error.WriteLine();
		}
	}

}
