﻿using System;
 
 
using System.Text;

using STMBench7.Core;

namespace STMBench7.Impl.Core
{

	/**
	 * STMBench7 benchmark Manual (see the specification).
	 * Default implementation.
	 */
	public class ManualImpl : Manual {

	    private readonly int id;
	    private String title;
	    private String text;
	    private Module module;

	    public ManualImpl(int id, String title, String text) {
	    	this.id = id;
	    	this.title = title;
	    	this.text = text;
	    }

	    public ManualImpl(ManualImpl source) {
	    	this.title = source.title;
	    	this.id = source.id;
	    	this.text = source.text;
	    	this.module = source.module;
	    }

        public virtual void SetModule(Module module)
        {
	    	this.module = module;
	    }

        public virtual int CountOccurences(char ch)
        {
	    	int position = 0, count = 0, newPosition, textLen = text.Length;

	    	do {
	    		newPosition = text.IndexOf(ch, position);
	    		if(newPosition == -1) break;

	    		position = newPosition + 1;
	    		count++;
	    	}
	    	while(position < textLen);

	    	return count;
	    }

        public virtual int CheckFirstLastCharTheSame()
        {
	    	if(text[0] == text[text.Length - 1]) 
                return 1;
	    	return 0;
	    }

        public virtual bool StartsWith(char ch)
        {
	    	return (text[0] == ch);
	    }

        public virtual int ReplaceChar(char from, char to)
        {
	    	text = text.Replace(from, to);
	    	return CountOccurences(to);
	    }

        public virtual int GetId()
        {
			return id;
		}

        public virtual Module GetModule()
        {
			return module;
		}

        public virtual String GetText()
        {
			return text;
		}

        public virtual String GetTitle()
        {
			return title;
		}
	}

}
