﻿using System;
 
 
using System.Text;
using STMBench7.Core;
using STMBench7.Compatability;
using System.Threading;

namespace STMBench7.Locking
{
	public class CGLockingOperationExecutor : OperationExecutor {

		private static readonly Lock globalReadLock, globalWriteLock;

        private static Object counterLock = new Object();
		private static int globalCounter = 0;

		static CGLockingOperationExecutor() {
			ReentrantReadWriteLock globalLock = new ReentrantReadWriteLock(true);
			globalReadLock = globalLock.ReadLock();
			globalWriteLock = globalLock.WriteLock();
		}

		private readonly Operation op;
		private readonly Lock globalLock;
        private int lastOperationTimestamp;
		
		public CGLockingOperationExecutor(Operation op) {
			this.op = op;
			if(op.GetOperationId() == null) {
				globalLock = null;
				return;
			}
		
			//switch(op.GetOperationId().GetType()) {
			OperationType t = op.GetOperationId().GetType();
            if (t == OperationType.OPERATION_RO || t == OperationType.SHORT_TRAVERSAL_RO || t == OperationType.TRAVERSAL_RO)
            {
            //case OPERATION_RO:
			//case SHORT_TRAVERSAL_RO:
			//case TRAVERSAL_RO: 
				globalLock = globalReadLock;
			//	break;
            }
            else if(t==OperationType.OPERATION || t==OperationType.SHORT_TRAVERSAL || t==OperationType.TRAVERSAL || t==OperationType.STRUCTURAL_MODIFICATION)
            {
			//case OPERATION:
			//case SHORT_TRAVERSAL:
			//case TRAVERSAL:
			//case STRUCTURAL_MODIFICATION:
				globalLock = globalWriteLock;
			//	break;
            }
            else {
			//default:
				throw new RuntimeError("Unexpected operation type");
			}
		}
		
		public int Execute() {
			try {
                if(globalLock != null) globalLock.Lock();
				return op.PerformOperation();
			}
			finally {
                lock (counterLock) { lastOperationTimestamp = globalCounter++; }
                if(globalLock != null) globalLock.Unlock();
			}
		}

		public int GetLastOperationTimestamp() {
			return lastOperationTimestamp;
		}
	}

}
