﻿using System;
 
 
using System.Text;

using STMBench7.Impl.Core;
using STMBench7.Core;
using STMBench7.Compatability;

namespace STMBench7.Locking
{
	/**
	 * An implementation of the ComplexAssembly interface used in the
	 * medium-grained locking synchronization method. The complex
	 * assembly is locked for reading or writing by the methods that read
	 * or modify the object (except for cases when the required
	 * locking is done externally).
	 */
	public class MGLockingComplexAssemblyImpl : ComplexAssemblyImpl {

		public MGLockingComplexAssemblyImpl(int id, String type, int buildDate,
                Module module, ComplexAssembly superAssembly)
            : base(id, type, buildDate, module, superAssembly)
        {
		}

		/*
		 * Methods "add/removeSubAssembly", "getSubAssemblies" and "clearPointers"
		 * assume global structure lock is held when those are called. Attributes
		 * "level", "superAssembly" and "module" can also be changed only when the
		 * global structure lock is held in write mode.
		 */

		public override int GetBuildDate() {
			MGLockingOperationExecutor.ReadLockAssemblyLevel(GetLevel());
			return base.GetBuildDate();
		}

		public override void UpdateBuildDate() {
			MGLockingOperationExecutor.WriteLockAssemblyLevel(GetLevel());
			base.UpdateBuildDate();
		}

		public override void NullOperation() {
            MGLockingOperationExecutor.ReadLockAssemblyLevel(GetLevel());
		}

		public override String GetType() {
			MGLockingOperationExecutor.ReadLockAssemblyLevel(GetLevel());
			return base.GetType();
		}
	}

}
