﻿using System;
 
 
using System.Text;
using STMBench7.Annotations;
using STMBench7.Core;
using STMBench7.Locking;

namespace STMBench7
{
	/**
	 * Creates an OperationExecutor object, which is used
	 * to execute the benchmark operations. For the default
	 * implementation, see stmbench7.impl.DefaultOperationExecutorFactory.
	 */
	[NonAtomic]
	public abstract class OperationExecutorFactory {

		public static OperationExecutorFactory instance = null;
		
		public static void SetInstance(OperationExecutorFactory newInstance) {
			if(instance != null)
				throw new RuntimeError("OperationExecutorFactory already instantiated");
			instance = newInstance;
		}
		
		public abstract OperationExecutor CreateOperationExecutor(Operation op);
		
		
        public static int ExecuteSequentialOperation(Operation op)
        {
		    OperationExecutor operationExecutor = instance.CreateOperationExecutor(op);
		    try
            {
                /**
                 *Ryan: Do sequential op outside of transaction for speed.
                 */
                if (operationExecutor is STMOperationExecutor) return op.PerformOperation();
                
                /**
                 *Ryan: But keep sequential op inside locks to ensure locks in modified data structure components are freed.
                 */
                else return operationExecutor.Execute();
		    }
		    catch(OperationFailedException) {
			    throw new RuntimeError("Unexpected failure of a sequential operation " + op);
		    }
	    }
	}

}
