﻿using System;
 
 
using System.Text;
using STMBench7.Annotations;
using STMBench7.Core;
using STMBench7.Backend;

namespace STMBench7.Operations
{
	/**
	 * Base class for all the benchmark operations.
	 */
	public abstract class BaseOperation : Operation {

        [Transactional]
        [Update]
        //public abstract int PerformOperation();
        public virtual int PerformOperation() { return 0; }

		public abstract OperationId GetOperationId();
		
		/**
		 * The method of adding and AtomicPart to the AtomicPartBuildDateIndex is
		 * non-trivial and is used in a few places. That is why it is put here for
		 * later reuse.
		 * 
		 * At first glance, it may seem more complicated than it is necessary,
		 * but that is because we want to use only locking implemented by
		 * an Index and Set implementations.
		 */
		public static void AddAtomicPartToBuildDateIndex(
				Index/*<IntIndexKey,LargeSet<AtomicPart>>*/ atomicPartBuildDateIndex,
				AtomicPart atomicPart) {
            LargeSet/*<AtomicPart>*/ newSet = BackendFactory.instance.CreateLargeSet/*<AtomicPart>*/();
			newSet.Add(atomicPart);
			LargeSet/*<AtomicPart>*/ sameDateSet = (LargeSet)
				atomicPartBuildDateIndex.PutIfAbsent(new IntIndexKey(atomicPart.GetBuildDate()), newSet);
			if(sameDateSet != null) sameDateSet.Add(atomicPart);
		}

		/**
		 * The method of removing and AtomicPart from the AtomicPartBuildDateIndex is
		 * non-trivial and is used in a few places. That is why it is put here for
		 * later reuse.
		 */
		public static void RemoveAtomicPartFromBuildDateIndex(
				Index/*<IntIndexKey,LargeSet<AtomicPart>>*/ atomicPartBuildDateIndex,
				AtomicPart atomicPart) {
			LargeSet/*<AtomicPart>*/ sameDateSet = (LargeSet)atomicPartBuildDateIndex.Get(new IntIndexKey(atomicPart.GetBuildDate()));
			sameDateSet.Remove(atomicPart);
		}
	}

}
