﻿using System;
 
 
using System.Text;
using STMBench7.Core;
using STMBench7.Compatability;
using STMBench7.Annotations;
using STMBench7.Backend;
using STMBench7.Impl.Backend;

namespace STMBench7.Operations
{
	/**
	 * Operation OP6 (see the specification).
	 * Read-only, search on index.
	 */
	public class Operation6 : BaseOperation {

		protected Index/*<IntIndexKey,ComplexAssembly>*/ complexAssemblyIdIndex;

		public Operation6(Setup oo7setup) {
			this.complexAssemblyIdIndex = oo7setup.GetComplexAssemblyIdIndex();
		}
		
		[Transactional] [ReadOnly]
        //[BartokSTMExtensions.NoLog]
        public override int PerformOperation()
        {
            
			int complexAssemblyId = ThreadRandom.NextInt(Parameters.MaxComplexAssemblies) +1;
			ComplexAssembly complexAssembly = (ComplexAssembly)complexAssemblyIdIndex.Get(new IntIndexKey(complexAssemblyId));
			if(complexAssembly == null) throw new OperationFailedException();
			
			ComplexAssembly superAssembly = complexAssembly.GetSuperAssembly();
			if(superAssembly == null) {
				PerformOperationInComplexAssembly((ComplexAssembly)complexAssembly);
				return 1;
			}
			
			int count = 0;


                                                        //cast so it doesn't bring in all implementers of GetEnumerator()
            foreach (Assembly siblingAssembly in ((ImmutableCollectionImpl)superAssembly.GetSubAssemblies()))
            {
				PerformOperationInComplexAssembly((ComplexAssembly)siblingAssembly);
				count++;
			}
			
			return count;
		}

        protected virtual void PerformOperationInComplexAssembly(ComplexAssembly assembly)
        {
			assembly.NullOperation();
		}
		
        public override OperationId GetOperationId()
        {
	    	return OperationId.OP6;
	    }
	}

}
