﻿using System;
 
 
using System.Text;
using STMBench7.Core;
using STMBench7.Annotations;
using STMBench7.Compatability;

namespace STMBench7.Operations
{
	/**
	 * Query Q6 (see the specification). Read-only, long traversal.
	 */
	public class Query6 : Query5 {

		protected Module module;

		public Query6(Setup oo7setup) : base(oo7setup) {
			this.module = oo7setup.GetModule();
		}

		[Transactional]
		[ReadOnly]
        [BartokSTMExtensions.NoLog]
        public override int PerformOperation()
        {
            return CheckComplexAssembly(module.GetDesignRoot());
		}

        [BartokSTMExtensions.NoLog]
        protected virtual int CheckAssembly(Assembly assembly)
        {
			if (assembly is BaseAssembly)
				return CheckBaseAssembly((BaseAssembly) assembly);
			else
				return CheckComplexAssembly((ComplexAssembly) assembly);
		}
        
        [BartokSTMExtensions.NoLog]
        protected virtual int CheckComplexAssembly(ComplexAssembly assembly)
        {
			int result = 0;

			foreach(Assembly subAssembly in assembly.GetSubAssemblies())
				result += CheckAssembly(subAssembly);

			if (result == 0)
				return 0;

			assembly.NullOperation();
			return result + 1;
		}
        
        public override OperationId GetOperationId()
        {
			return OperationId.Q6;
		}
	}

}
