﻿using System;
 
 
using System.Text;
using STMBench7.Core;
using STMBench7.Annotations;

namespace STMBench7.Operations
{
	/**
	 * An operation that initializes the benchmark data structure.
	 */
	public class SetupDataStructure : Operation {

		private readonly Setup setup;
		private Module module;
		
		public SetupDataStructure(Setup setup) {
			this.setup = setup;
		}

		[Update]
        [BartokSTMExtensions.NoLogTrans] /* We don't want to wait for this long transaction that is not even benchmarked */
        public virtual int PerformOperation()
        {
	    	Console.Error.WriteLine("Setting up the design library:");
	    	CompositePart[] designLibrary = new CompositePart[Parameters.InitialTotalCompParts];
	    	CompositePartBuilder compositePartBuilder = setup.GetCompositePartBuilder();
	    	
	    	for(int i = 0; i < Parameters.InitialTotalCompParts; i++) {
	    	    Console.Error.Write("Component " + (i+1) + " of " + Parameters.InitialTotalCompParts + "\r");
	    	    try {
	    	    	designLibrary[i] = compositePartBuilder.CreateAndRegisterCompositePart();
	    	    }
	    	    catch(OperationFailedException e) {
	    	    	throw new RuntimeError("Unexpected failure of createAndRegisterCompositePart!", e);
	    	    }
	    	}
	    	Console.Error.WriteLine();

	    	Console.Error.WriteLine("Setting up the module:");
	    	try {
	    		module = setup.GetModuleBuilder().CreateRegisterModule();
	    	}
	    	catch(OperationFailedException e) {
	    		throw new RuntimeError("Unexpected failure of createRegisterModule!", e);
	    	}
	    	
	    	int j = 1;
	    	foreach(BaseAssembly baseAssembly in setup.GetBaseAssemblyIdIndex()) {
	    		Console.Error.Write("Base Assembly " + (j++) + " of " + Parameters.InitialTotalBaseAssemblies + "\r");
	    		
	    		for(int connections = 0; connections < Parameters.NumCompPerAssm; connections++) {
	    			int compositePartNum = ThreadRandom.NextInt(designLibrary.Length);
	    			baseAssembly.AddComponent(designLibrary[compositePartNum]);
	    		}
	    	}
            
            return 0;
		}

        public virtual OperationId GetOperationId()
        {
			return null;
		}

        public virtual Module GetModule()
        {
			return module;
		}
	}

}
