﻿using System;
 
 
using System.Text;
using STMBench7.Compatability;
using STMBench7.Core;

namespace STMBench7.Operations
{
	/**
	 * Short traversal ST9 (see the specification).
	 * Read-only, short.
	 */
	public class ShortTraversal9 : ShortTraversal1 {

        public ShortTraversal9(Setup oo7setup)
            : base(oo7setup)
        {
		}
		
        [BartokSTMExtensions.NoLog]
        protected override int Traverse(CompositePart component)
        {
            HashtableSet/*<AtomicPart>*/ setOfVisitedPartIds = new HashtableSet/*<AtomicPart>*/();
			return Traverse(component.GetRootPart(), setOfVisitedPartIds);
		}

        protected virtual int Traverse(AtomicPart atomicPart, HashtableSet/*<AtomicPart>*/ setOfVisitedPartIds)
        {
			if(atomicPart == null) return 0;
			if(setOfVisitedPartIds.Contains(atomicPart)) return 0;

			int result = PerformOperationInAtomicPart(atomicPart);

			setOfVisitedPartIds.Add(atomicPart);

			foreach(Connection connection in atomicPart.GetToConnections())
				result += Traverse(connection.GetDestination(), setOfVisitedPartIds);

			return result;
		}

        [BartokSTMExtensions.NoLog]
        protected virtual int PerformOperationInAtomicPart(AtomicPart part)
        {
			part.NullOperation();
			return 1;
		}

        [BartokSTMExtensions.NoLogTrans]
        protected override int Traverse(AtomicPart atomicPart)
        {
			throw new RuntimeError("ST9: unexpected call to traverse(AtomicPart)!");
		}


		
        public override OperationId GetOperationId()
        {
	    	return OperationId.ST9;
	    }
	}

}
