﻿using System;
 
 
using System.Text;
using STMBench7.Core;
using STMBench7.Backend;
using STMBench7.Annotations;
using STMBench7.Compatability;

namespace STMBench7.Operations
{
	/**
	 * Structural modification operation SM3 (see the specification).
	 */
	public class StructuralModification3 : BaseOperation {

		protected Index/*<IntIndexKey,BaseAssembly>*/ baseAssemblyIdIndex;
		protected Index/*<IntIndexKey,CompositePart>*/ compositePartIdIndex;
		
		public StructuralModification3(Setup oo7setup) {
		
			this.baseAssemblyIdIndex = oo7setup.GetBaseAssemblyIdIndex();
			this.compositePartIdIndex = oo7setup.GetCompositePartIdIndex();
		}
		
		[Transactional] [Update]
        [BartokSTMExtensions.NoLog]
        public override int PerformOperation()
        {
            int baseAssemblyId = ThreadRandom.NextInt(Parameters.MaxBaseAssemblies) + 1;
			int componentId = ThreadRandom.NextInt(Parameters.MaxCompParts) + 1;
			BaseAssembly baseAssembly = (BaseAssembly)baseAssemblyIdIndex.Get(new IntIndexKey(baseAssemblyId));
			CompositePart component = (CompositePart)compositePartIdIndex.Get(new IntIndexKey(componentId));
			
			if(baseAssembly == null || component == null) throw new OperationFailedException();
			
			baseAssembly.AddComponent(component);
			
			return 0;
		}
		
        public override OperationId GetOperationId()
        {
	    	return OperationId.SM3;
	    }
	}

}
