﻿using System;
 
 
using System.Text;
using STMBench7.Core;
using STMBench7.Compatability;
using STMBench7.Annotations;

namespace STMBench7.Operations
{
	/**
	 * Traversal T1 (see the specification).
	 * Read-only, long.
	 */
	public class Traversal1 : BaseOperation {

		protected Module module;

		public Traversal1(Setup oo7setup) {
			this.module = oo7setup.GetModule();
		}

		[Transactional] [ReadOnly]
        [BartokSTMExtensions.NoLog]
        public override int PerformOperation()
        {
            ComplexAssembly designRoot = module.GetDesignRoot();
			return Traverse(designRoot);
		}

        [BartokSTMExtensions.NoLog]
        protected virtual int Traverse(Assembly assembly)
        {
			if(assembly is BaseAssembly) return Traverse((BaseAssembly)assembly);
			else return Traverse((ComplexAssembly)assembly);
		}

        [BartokSTMExtensions.NoLog]
        protected virtual int Traverse(ComplexAssembly complexAssembly)
        {
			int partsVisited = 0;

			foreach(Assembly assembly in complexAssembly.GetSubAssemblies())
				partsVisited += Traverse(assembly);

			return partsVisited;
		}

        [BartokSTMExtensions.NoLog]
        protected virtual int Traverse(BaseAssembly baseAssembly)
        {
			int partsVisited = 0;

			foreach(CompositePart component in baseAssembly.GetComponents())
				partsVisited += Traverse(component);

			return partsVisited;
		}

        [BartokSTMExtensions.NoLog]
        protected virtual int Traverse(CompositePart component)
        {
			AtomicPart rootPart = component.GetRootPart();
            HashtableSet/*<AtomicPart>*/ setOfVisitedPartIds = new HashtableSet/*<AtomicPart>*/();
        
			return Traverse(rootPart, setOfVisitedPartIds);
		}

        [BartokSTMExtensions.NoLog]
        protected virtual int Traverse(AtomicPart part, HashtableSet/*<AtomicPart>*/ setOfVisitedPartIds)
        {
			if(part == null) return 0;
			if(setOfVisitedPartIds.Contains(part)) return 0;

			int result = PerformOperationInAtomicPart(part, setOfVisitedPartIds);

			setOfVisitedPartIds.Add(part);

			foreach(Connection connection in part.GetToConnections())
				result += Traverse(connection.GetDestination(), setOfVisitedPartIds);

			return result;
		}

        [BartokSTMExtensions.NoLog]
        protected virtual int PerformOperationInAtomicPart(AtomicPart part, HashtableSet/*<AtomicPart>*/ setOfVisitedPartIds)
        {
			part.NullOperation();
			return 1;
		}
		
        public override OperationId GetOperationId()
        {
	    	return OperationId.T1;
	    }
	}

}
